from distutils.core import setup
setup(
  name = 'pom3',
  packages = ['pom3'],
  version = '1.0',
  description = 'Simulation of agile requirements engineering.',
  author = 'Joseph Krall',
  author_email = 'kralljoe@gmail.com',
  url = 'https://github.com/tanzairatier/pom3', 
  download_url = 'https://github.com/tanzairatier/pom3/archive/1.0.tar.gz',
  keywords = ['agile', 'requirements', 'engineering', 'simulation', 'jmoo'],
  classifiers = [],
)